#pragma once
#ifndef PERFORMANCEMONITOR_H
#define PERFORMANCEMONITOR_H
/*-----------------------------------------------------------------------------

Performance Monitor.

Creating a performance monitor object will start a sampling session. Samples
taken within other sessions are built as a tree structure. 

-----------------------------------------------------------------------------*/

#define PERFORMANCE_MONITOR_ENABLED

/*-----------------------------------------------------------------------------
Includes for Interface
-----------------------------------------------------------------------------*/
#include <string>
#include "Tree.h"
#include "Timer.h"


/*-----------------------------------------------------------------------------
Class
-----------------------------------------------------------------------------*/
class CPerformanceStatistics
{
public:

	CPerformanceStatistics(const std::string& a_Name);
	bool operator== (const CPerformanceStatistics& a_Stats) const;

public:

	std::string		Name;
	float			Percent;
	unsigned int	Min;
	unsigned int	Max;
	unsigned int	TotalTime;
	unsigned int	Samples;

};


class CPerformanceMonitor
{
public:

	CPerformanceMonitor(const char* a_Name);
	~CPerformanceMonitor();

	static void Reset(void);

public:

	static tree<CPerformanceStatistics>						Statistics;

private:

	CTimer													m_Timer;
	tree<CPerformanceStatistics>::iterator					m_ThisNode;
	static tree<CPerformanceStatistics>::sibling_iterator	s_Parent;

};


/*-----------------------------------------------------------------------------
Inlined Functions
-----------------------------------------------------------------------------*/
#include "PerformanceMonitor.inl"

#ifdef PERFORMANCE_MONITOR_ENABLED
	#define Profile(Name) CPerformanceMonitor PerformanceMonitor(Name)
#else
	#define Profile(Name)
#endif


#endif //PERFORMANCEMONITOR_H